/**
 * DDSAppletChild.java.
 */
package dds.control;

/**
 * The DDSAppletChild class is designed to provide a basic functionality for
 * classes that operate directly under a DDSApplet. The reason for this class is
 * to establish a simple child-parent relstionship that allows the children of a
 * DDSApplet communicate directly with it via a stored reference within the
 * child class. This should alleviate the need to use long complicated method
 * chains to access the DDSApplet. Only DDSAppletChildren should be able to
 * access the DDSApplet directly. This also serves to establish a functional
 * hierarchy of classes which manage the various aspects of this application.
 * 
 * @author Jacob Bolton
 */
public abstract class AbstractDDSAppletChild {

    /**
     * The NO_APPLET constant is used to initialize and possibly later set the
     * parent to reference to null.
     */
    private static final DDSApplet NO_APPLET = null;

    /**
     * The parent field is the reference to the parent DDSApplet, which is used
     * to communicate directly back to the DDSApplet.
     */
    private DDSApplet parent;

    /**
     * Private constructor for DDSAppletChild. It just sets the parent to null.
     */
    private AbstractDDSAppletChild() {
        super();
        
        // Set parent to null so that it's been initialized.
        this.parent = NO_APPLET;
    }

    /**
     * Constructor for AbstractDDSAppletChild that allows the parent to be set.
     * This should be the only constructor used to create DDSAppletChildren.
     * 
     * @param applet The parent DDSApplet for this child.
     */
    public AbstractDDSAppletChild(final DDSApplet applet) {
        this.parent = applet;
    }

    /**
     * Getter for parent.
     * 
     * @return the parent
     */
    public final DDSApplet getParent() {
        return parent;
    }

    /**
     * Setter for parent.
     * 
     * @param applet the parent to set
     */
    public final void setParent(final DDSApplet applet) {
        this.parent = applet;
    }

    /**
     * The destruct method is used to destroy this object when the DDSApplet
     * parent is done with it to allow for easier garbage collection. It does
     * this by unsetting the references this object has to/with other objects.
     */
    public abstract void destruct();

    /**
     * The setup method is used to perform any necessary operations that this
     * objects needs to execute in order to ready itself for proper use by the
     * DDSApplet parent.
     */
    public abstract void setup();

}
