/**
 * DDSImage.java.
 */
package dds.control;

import processing.core.PImage;

/**
 * The DDSImage class is a wrapper for PImage that give public access to the
 * loadTIFF and checkAlpha methods in PImage. It exists as part of a solution
 * that allows images to be loaded from the data repository via the
 * ResourceLoader class. The current solution depends on this class for access
 * to certain methods, however there is no extra or custom functionality being
 * provided. This indeicates that this class can be removed and the
 * functionality it provides be converted into method calls via reflection.
 * Therefore, this class is not a permanent fixture.
 * 
 * @author Jacob Bolton
 */
public class DDSImage extends PImage {

    /**
     * Constructor for DDSImage.
     * 
     * @param pimage The PImage to give specialized functionality to.
     */
    public DDSImage(final PImage pimage) {
        super();
        // Copy over all fields from the original PImage
        this.cache = pimage.cache;
        this.format = pimage.format;
        this.height = pimage.height;
        this.imageMode = pimage.imageMode;
        this.modified = pimage.modified;
        this.mx1 = pimage.mx1;
        this.mx2 = pimage.mx2;
        this.my1 = pimage.my1;
        this.my2 = pimage.my2;
        this.parent = pimage.parent;
        this.pixels = pimage.pixels;
        this.smooth = pimage.smooth;
        this.width = pimage.width;
        // Since this now has all the information of the original PImage, that
        // PImage no longer needs to be referenced and can be garbage collected
        // when convenient.
    }

    /**
     * The publicCheckAlpha method wraps the protected superclass method
     * checkAlpha. This is used when loading GIF and PNG (and other
     * transparency-capable formats) from the data repository.
     */
    public final void publicCheckAlpha() {
        // Call the protected checkAlpha method from Processing.
        super.checkAlpha();
    }

    /**
     * The publicLoadTIFF method wraps the protected superclass method loadTIFF.
     * This is used when loading formats from the data repository.
     * 
     * @param arg0 The byte[] containing TIFF image data.
     * @return The loaded PImage.
     */
    // TODO Make sure that Processing's loadTIFF method will actually work with
    // out repository implementation.
    public static final PImage publicLoadTIFF(final byte[] arg0) {
        return PImage.loadTIFF(arg0);
    }
}
