/**
 * This java class is distributed under the BSD license. Copyright 2005 Lilian
 * Chamontin. contact lilian.chamontin at f r e e . f r
 */

/**
 * Portions Copyright (c) 2003 Sun Microsystems, Inc. All Rights Reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: -
 * Redistribution of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer. - Redistribution in binary
 * form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided
 * with the distribution. Neither the name of Sun Microsystems, Inc. or the
 * names of contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission. This software is
 * provided "AS IS," without a warranty of any kind. ALL EXPRESS OR IMPLIED
 * CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. SUN MICROSYSTEMS, INC. ("SUN") AND ITS LICENSORS SHALL NOT
 * BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
 * MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL
 * SUN OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR
 * DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES,
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE
 * USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES. You acknowledge that this software is not
 * designed or intended for use in the design, construction, operation or
 * maintenance of any nuclear facility.
 */
package dds.control;

import java.applet.Applet;
import java.applet.AppletContext;
import java.applet.AppletStub;
import java.applet.AudioClip;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

/**
 * Basic JOGL installer for Applets. The key functionality this class supplies
 * is the ability to deploy unsigned applets which use JOGL. It may also be used
 * to deploy signed applets in which case multiple security dialogs will be
 * displayed.
 * <p>
 * On the server side the codebase must contain jogl.jar and all of the
 * jogl-natives-*.jar files from the standard JOGL distribution. This is the
 * location from which the JOGL library used by the applet is downloaded. The
 * codebase additionally contains the jar file of the user's potentially
 * untrusted applet. The jogl.jar and all jogl-natives jars must be signed by
 * the same entity, which is typically Sun Microsystems, Inc.
 * <p>
 * There are some limitations with this approach. It is not possible to specify
 * e.g. -Dsun.java2d.noddraw=true or -Dsun.java2d.opengl=true for better control
 * over the Java2D pipeline as it is with Java Web Start. There appear to be
 * issues with multiple JOGL-based applets on the same web page, though multiple
 * instances of the same applet appear to work. The latter may simply be a bug
 * which needs to be fixed.
 * <p>
 * The JOGL natives are cached in the user's home directory (the value of the
 * "user.home" system property in Java) under the directory .jogl_ext. The Java
 * Plug-In is responsible for performing all other jar caching. If the JOGL
 * installation is updated on the server, the .jogl_ext cache will automatically
 * be updated.
 * <p>
 * This technique requires that JOGL has not been installed in to the JRE under
 * e.g. jre/lib/ext. If problems are seen when deploying this applet launcher,
 * the first question to ask the end user is whether jogl.jar and any associated
 * DLLs, .so's, etc. are installed directly in to the JRE. The applet launcher
 * has been tested primarily under Mozilla and Firefox; there may be problems
 * when running under, for example, Opera.
 * <p>
 * 
 * @author Lilian Chamontin
 * @author Kenneth Russell
 */
/**
 * The DDSLauncher class is responsible for running the Defensive Driving
 * Simulator application in a stand-alone fashion or via Java WebStart. This is
 * accomplished by extending the JOGLAppletLauncher class, which usually loads
 * the applet on a webpage. The extension changes the default operation so that
 * the applet will seek its resources from within a single jar file.
 * 
 * @author Jacob Bolton
 */
public class DDSLauncher extends Applet implements AppletStub, AppletContext {

    /**
     * The EIGHT_KB constant is the nuber of bytes needed to make 8kb.
     */
    private static final int EIGHT_KB = 8192;

    /**
     * The HUNDRED constant is the number 100.
     */
    private static final int HUNDRED = 100;

    /**
     * The ONE_KB constant is the nuber of bytes needed to make 1kb.
     */
    private static final int ONE_KB = 1024;

    /**
     * The serialVersionUID.
     */
    private static final long serialVersionUID = 1L;

    static {
        // TODO Determine if we really want to do this.
        // Try to set a look consistent with the system's settings.
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception ignore) {
            // TODO Add logger.
            System.err.println(ignore);
        }
    }

    /**
     * The main method would be responsible for running the Defensive Driving
     * Simulator as a stand-alone application outside of a web-browser. This
     * method should start up an applet viewer and load an instance of DDSApplet
     * into it.
     * 
     * @param args Command line args.
     */
    public static void main(final String[] args) {
        // TODO Consider parsinng args for things like the remote repository
        // location.

        // TODO Determine if this actually helps anything.
        // System.setProperty("JOGL_SINGLE_THREADED_WORKAROUND", "true");

        // Create a new launcher.
        final DDSLauncher launcher = new DDSLauncher();

        // The DDSLauncher will act as its own AppletStub.
        launcher.setStub(launcher);

        // Initialize the DDSLauncher.
        launcher.init();

        // Start the DDSLauncher.
        launcher.start();
    }

    /**
     * The appletFrame field contains a reference to the JFrame that contains
     * the applet. This is mainly for size commands coming from within the
     * applet.
     */
    private final transient JFrame appletFrame;

    /**
     * The base field stores the URL for the base directory of operation for the
     * DDSLauncher.
     */
    private transient URL base;

    /**
     * The driHackPresent field indicates whether the "DRI hack" native library
     * is present and whether we therefore might need to run the DRIHack during
     * loading of the native libraries.
     */
    private transient boolean driHackPresent;

    /**
     * The firstStart field indicates if this is the first time this has been
     * initialized. Set to false once start() has been invoked.
     */
    private transient boolean firstStart;

    /**
     * The installDirectory field defines a private directory for native libs.
     */
    private transient String installDirectory;

    /**
     * The isInitOk field indicates if this has init'ed properly.
     */
    private transient boolean isInitOk = false;

    /**
     * The joglStarted field indicates if the GL components have initialized
     * properly. Set to true if start() has passed successfully.
     */
    private transient boolean joglStarted = false;

    /**
     * The loaderPanel is the JPanel which will contain the applet.
     */
    // TODO Move instantiating of this to the constructor.
    private final transient JPanel loaderPanel = new JPanel(new BorderLayout());

    /**
     * The loadingImage field contains a reference to the URL for the location
     * of the image that is to be shown as the GL classes are initialized.
     */
    private final transient URL loadingImage;

    /**
     * The nativeLibInfo stores the info for the native libraries selected for
     * this system configuration.
     */
    private transient NativeLibInfo nativeLibInfo;

    /**
     * The nativeLibNames field stores the library names, which are computed
     * once the jar comes down. The signatures of these native libraries are NOT
     * checked before installing them. This is because we decided to cut some
     * corners since we're packaging these libraries within the application jar
     * and not serving them from the web.
     */
    private transient String[] nativeLibNames;

    /**
     * The progressBar is displayed during the loading of GL libraries.
     */
    // TODO Move instantiating of this to the constructor.
    private final transient JProgressBar progressBar = new JProgressBar(0, 100);

    /**
     * The subApplet field stores a reference to the applet we have to start.
     */
    private transient Applet subApplet;

    /**
     * Constructor for DDSLauncher.
     */
    public DDSLauncher() {
        super();

        // // Print out the system properties to assist in development.
        // System.out.println(System.getProperties());

        // Set up the JFrame used to contain the application applet.
        this.appletFrame = new JFrame("Defensive Driving Simulator");
        this.appletFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // this.appletFrame.setSize(new Dimension(DDSApplet.DEFAULT_WIDTH,
        // DDSApplet.DEFAULT_HEIGHT));
        this.appletFrame.setPreferredSize(new Dimension(
                DDSApplet.DEFAULT_WIDTH, DDSApplet.DEFAULT_HEIGHT));

        // Add this launcher to the frame so it can be displayed.
        // TODO Maybe do this later?
        this.appletFrame.add(this);

        // Set a preferred size for this applet.
        // TODO Determine if this is necessary.
        // setSize(new Dimension(DDSApplet.DEFAULT_WIDTH,
        // DDSApplet.DEFAULT_HEIGHT));
        setPreferredSize(new Dimension(DDSApplet.DEFAULT_WIDTH,
                DDSApplet.DEFAULT_HEIGHT));

        this.appletFrame.setBackground(Color.BLACK);

        // Get a reference to the loading image as a URL.
        URL imageURL = this.getClass().getResource("/data/images/loading.png");

        if (imageURL == null) {
            try {
                this.base = new URL("file:" + System.getProperty("user.dir")
                        + "/");
                imageURL = new URL("file:" + System.getProperty("user.dir")
                        + "/data/images/loading.png");
            } catch (MalformedURLException e) {

                // TODO Add logger.
                System.err.println(e);
            }
        } else {
            try {
                this.base = new URL(imageURL.toString().replace(
                        "/data/images/loading.png", "/"));
                System.out.println("base = " + base);
            } catch (MalformedURLException e) {
                // TODO Add Logger
                System.err.println(e);
            }
        }
        this.loadingImage = imageURL;

        this.firstStart = true;

    }

    /**
     * Called when the applet wants to be resized.
     * 
     * @param width The new requested width for the applet.
     * @param height The new requested height for the applet.
     * @see java.applet.AppletStub#appletResize(int, int)
     */
    public final void appletResize(final int width, final int height) {
        // Resize the JFrame container.
        this.appletFrame.setSize(width, height);
        this.appletFrame.setPreferredSize(new Dimension(width, height));
    }

    /**
     * Called by the browser or applet viewer to inform this applet that it is
     * being reclaimed and that it should destroy any resources that it has
     * allocated. The <code>stop</code> method will always be called before
     * <code>destroy</code>.
     * <p>
     * A subclass of <code>Applet</code> should override this method if it has
     * any operation that it wants to perform before it is destroyed. For
     * example, an applet with threads would use the <code>init</code> method
     * to create the threads and the <code>destroy</code> method to kill them.
     * <p>
     * The implementation of this method provided by the <code>Applet</code>
     * class does nothing.
     * 
     * @see java.applet.Applet#init()
     * @see java.applet.Applet#start()
     * @see java.applet.Applet#stop()
     * @see java.applet.Applet#destroy()
     */
    public final void destroy() {
        // Make sure we have a subApplet to destroy.
        if (subApplet != null) {
            // Destroy the subApplet.
            subApplet.destroy();
        }
    }

    /**
     * The displayError method displays an error message on the progress bar.
     * 
     * @param errorMessage The message to display.
     */
    private void displayError(final String errorMessage) {
        // TODO Consider logging this.
        progressBar.setString("Error : " + errorMessage);
    }

    /**
     * The displayMessage method displays an informational message on the
     * progress bar.
     * 
     * @param message The message to display.
     */
    private void displayMessage(final String message) {
        // TODO Consider logging this.
        progressBar.setString(message);
    }

    /**
     * The findNativeEntries method detects the native libraries within the jar
     * file.
     * 
     * @param jarFile The jar file to scan for libraries.
     * @return True if native libraries were found in the jar.
     */
    private boolean findNativeEntries(final JarFile jarFile) {
        final List<String> list = new ArrayList<String>();
        final Enumeration entries = jarFile.entries();
        while (entries.hasMoreElements()) {
            final JarEntry entry = (JarEntry) entries.nextElement();
            if (nativeLibInfo.matchesNativeLib(entry.getName())) {
                list.add(entry.getName());
                if (entry.getName().indexOf("jogl_drihack") >= 0) {
                    driHackPresent = true;
                }
            }
        }
        boolean result = true;
        if (list.isEmpty()) {
            result = false;
        }
        nativeLibNames = (String[]) list.toArray(new String[list.size()]);
        return result;
    }

    /**
     * Finds and returns the applet in the document represented by this applet
     * context with the given name. The name can be set in the HTML tag by
     * setting the <code>name</code> attribute. In DDSLauncher, this will
     * always return the DDSLauncher regardless of what name is passed in. This
     * because the DDSLauncher will be the only Applet within it's context an it
     * doesn't need to be named.
     * 
     * @param name an applet name.
     * @return the applet with the given name, or <code>null</code> if not
     *         found.
     * @see java.applet.AppletContext#getApplet(java.lang.String)
     */
    public final Applet getApplet(final String name) {
        return this;
    }

    /**
     * Gets a handler to the applet's context, which is always the DDSLauncher
     * itself.
     * 
     * @return the applet's context.
     * @see java.applet.AppletStub#getAppletContext()
     */
    public final AppletContext getAppletContext() {
        return this;
    }

    /**
     * Finds all the applets in the document represented by this applet context.
     * However, the DDSLauncher will always only contain itself.
     * 
     * @return an enumeration of all applets in the document represented by this
     *         applet context.
     * @see java.applet.AppletContext#getApplets()
     */
    public final Enumeration<Applet> getApplets() {
        // TODO Write getApplets in DDSLauncher.
        new UnsupportedOperationException(
                "Need to write DDSLauncher.getApplets");
        System.out.println("getApplets");
        return null;
    }

    /**
     * Returns the <code>AudioClip</code> object specified by the
     * <code>URL</code> argument.
     * <p>
     * This method always returns immediately, whether or not the audio clip
     * exists. When this applet attempts to play the audio clip, the data will
     * be loaded.
     * 
     * @param url an absolute URL giving the location of the audio clip.
     * @return the audio clip at the specified URL.
     * @see java.applet.AudioClip
     * @see java.applet.Applet#getAudioClip(java.net.URL)
     */
    public final AudioClip getAudioClip(final URL url) {
        // TODO Write in getAudioClip DDSLauncher.
        new UnsupportedOperationException(
                "Need to write DDSLauncher.getAudioClip");
        System.out.println("getAudioClip");
        return null;
    }

    /**
     * Gets the base URL. This is the URL of the directory which contains the
     * applet. The DDSLauncher should always return a URL pointing to the jar it
     * is contained in, unless is is being run from Eclipse.
     * 
     * @return the base {@link java.net.URL} of the directory which contains the
     *         applet.
     * @see java.applet.AppletStub#getDocumentBase()
     * @see java.applet.AppletStub#getCodeBase()
     */
    public final URL getCodeBase() {
        return this.base;
    }

    /**
     * Gets the URL of the document in which the applet is embedded. For
     * example, suppose an applet is contained within the document: <blockquote>
     * http://java.sun.com/products/jdk/1.2/index.html </blockquote> The
     * document base is: <blockquote>
     * http://java.sun.com/products/jdk/1.2/index.html </blockquote>
     * 
     * @return the {@link java.net.URL} of the document that contains the
     *         applet.
     * @see java.applet.AppletStub#getCodeBase()
     * @see java.applet.AppletStub#getDocumentBase()
     */
    public final URL getDocumentBase() {
        // TODO Make sure this is correct functionality.
        return this.getClass().getResource("/");
    }

    /**
     * Returns an <code>Image</code> object that can then be painted on the
     * screen. The <code>url</code> that is passed as an argument must specify
     * an absolute URL.
     * <p>
     * This method always returns immediately, whether or not the image exists.
     * When this applet attempts to draw the image on the screen, the data will
     * be loaded. The graphics primitives that draw the image will incrementally
     * paint on the screen.
     * 
     * @param url an absolute URL giving the location of the image.
     * @return the image at the specified URL.
     * @see java.awt.Image
     * @see java.applet.Applet#getImage(java.net.URL)
     */
    public final Image getImage(final URL url) {
        // TODO Write getImage in DDSLauncher.
        new UnsupportedOperationException("Need to write DDSLauncher.getImage");
        System.out.println("getImage");
        return null;
    }

    /**
     * Returns the value of the named parameter in the HTML tag. For example, if
     * an applet is specified as <blockquote> </blockquote>
     * <p>
     * then a call to <code>getParameter("Color")</code> returns the value
     * <code>"blue"</code>.
     * 
     * @param name a parameter name.
     * @return the value of the named parameter, or <tt>null</tt> if not set.
     * @see java.applet.AppletStub#getParameter(java.lang.String)
     */
    public final String getParameter(final String name) {
        String result = null;
        if ("subapplet.classname".equals(name)
                || "subapplet.displayname".equals(name)) {
            result = DDSApplet.class.getName().toString();

        } else if ("subapplet.image".equals(name)) {
            result = loadingImage.toString();
        } else if ("width".equals(name)) {
            result = Integer.toString(DDSApplet.DEFAULT_WIDTH);
        } else if ("height".equals(name)) {
            result = Integer.toString(DDSApplet.DEFAULT_HEIGHT);
        }
        return result;
    }

    /**
     * Returns the stream to which specified key is associated within this
     * applet context. Returns <tt>null</tt> if the applet context contains no
     * stream for this key.
     * <p>
     * For security reasons, mapping of streams and keys exists for each
     * codebase. In other words, applet from one codebase cannot access the
     * streams created by an applet from a different codebase
     * <p>
     * 
     * @return the stream to which this applet context maps the key
     * @param key key whose associated stream is to be returned.
     * @since JDK1.4
     * @see java.applet.AppletContext#getStream(java.lang.String)
     */
    public final InputStream getStream(final String key) {
        // TODO Write getStream in DDSLauncher.
        new UnsupportedOperationException(
                "Need to write DDSLauncher.getStream()");
        System.out.println("getStream");
        return null;
    }

    /**
     * Finds all the keys of the streams in this applet context.
     * <p>
     * For security reasons, mapping of streams and keys exists for each
     * codebase. In other words, applet from one codebase cannot access the
     * streams created by an applet from a different codebase
     * <p>
     * 
     * @return an Iterator of all the names of the streams in this applet
     *         context.
     * @since JDK1.4
     * @see java.applet.AppletContext#getStreamKeys()
     */
    public final Iterator<String> getStreamKeys() {
        // TODO Write getStreamKeys in DDSLauncher.
        new UnsupportedOperationException(
                "Need to write DDSLauncher.getStreamKeys");
        System.out.println("getStreamKeys");
        return null;
    }

    /**
     * The getSubApplet method is a helper method that makes it easier to call
     * methods on the sub-applet from JavaScript.
     * 
     * @return The subApplet.
     */
    public final Applet getSubApplet() {
        return subApplet;
    }

    /**
     * The init method performs applet initialization.
     * 
     * @see java.applet.Applet#init()
     */
    public final void init() {
        // Setup the layout of the panel.
        initLoaderLayout();

        // Validate the components.
        // TODO Determine if this is necessary.
        validate();

        // Try to resolve the current platform against the ones that JOGL
        // supports.
        this.nativeLibInfo = NativeLibInfo.detectPlatform();

        // Just use "dds" as the installation folder rather than use a really
        // long name derived from the codebase.
        this.installDirectory = "dds_"
                + this.nativeLibInfo.toString()
                        .toLowerCase(Locale.getDefault());

        // Check OS and architecture against the ones supported.
        this.isInitOk = this.nativeLibInfo.isSupported();

        // Verify that the current system configuration is supported by native
        // libraries.
        if (!this.isInitOk) {
            // Get the system's OS and architecture.
            displayError("Init failed : Unsupported os / arch ( "
                    + System.getProperty("os.name") + " / "
                    + System.getProperty("os.arch") + " )");
        }
    }

    /**
     * The initLoaderLayout method lays out the initial loading screen for the
     * application. This is displayed as the GL is installed and initialized.
     */
    private void initLoaderLayout() {
        // TODO Have this layout the JFrame?

        // Attach a layout.
        setLayout(new BorderLayout());

        // Setup the progress bar.
        progressBar.setBorderPainted(true);
        progressBar.setStringPainted(true);
        progressBar.setString("Loading...");

        // Prepare the loading image.
        ImageIcon image = null;
        image = new ImageIcon(this.loadingImage);

        // Add the loaderPanel.
        add(loaderPanel, BorderLayout.SOUTH);

        // check to see if we were able to load the loading image.
        if (image != null) {
            // We were so add it.
            loaderPanel.add(new JLabel(image), BorderLayout.CENTER);
        }

        // Add the progress bar.
        loaderPanel.add(progressBar, BorderLayout.SOUTH);
    }

    /**
     * The installFile method saves a file from the specified jar to the
     * specified directory.
     * 
     * @param installDir The installation directory.
     * @param jar The jar file.
     * @param fileName The name of the file to extract and save.
     * @param buf The byte buffer to use.
     * @return True if operation was successful.
     */
    private boolean installFile(final File installDir, final JarFile jar,
            final String fileName, final byte[] buf) {
        boolean result = true;
        try {
            final JarEntry entry = jar.getJarEntry(fileName);
            if (entry == null) {
                displayError("Error finding native library " + fileName);
                result = false;
            } else {
                final InputStream inStream = jar.getInputStream(entry);
                // int totalLength = (int) entry.getSize();
                BufferedOutputStream out = null;
                final File outputFile = new File(installDir, fileName);
                try {
                    out = new BufferedOutputStream(new FileOutputStream(
                            outputFile));
                } catch (Exception e) {
                    displayError("Error opening file " + fileName
                            + " for writing");
                    result = false;
                }
                if (result) {
                    try {
                        int len = inStream.read(buf);
                        while (len > 0) {
                            out.write(buf, 0, len);
                            len = inStream.read(buf);
                        }
                        out.flush();
                        out.close();
                    } catch (IOException ioe) {
                        // TODO Add logger or put logging in displayError.
                        System.err.println(ioe);
                        displayError("Error writing file " + fileName
                                + " to disk");
                        outputFile.delete();
                        result = false;
                    }
                }
            }
        } catch (Exception e2) {
            // TODO Add logger or put logging in displayError.
            System.err.println(e2);
            displayError("Error writing file " + fileName + " to disk");
            result = false;
        }
        return result;
    }

    /**
     * Determines if the applet is active. An applet is active just before its
     * <code>start</code> method is called. It becomes inactive just before
     * its <code>stop</code> method is called.
     * 
     * @return <code>true</code> if the applet is active; <code>false</code>
     *         otherwise.
     * @see java.applet.AppletStub#isActive()
     */
    public final boolean isActive() {
        // TODO Have this return false after activation (start)?
        return true;
    }

    /**
     * The loadLibrary method attempts to load the specified GL library.
     * 
     * @param installDir The installation path.
     * @param libName The library name.
     */
    private void loadLibrary(final File installDir, final String libName) {
        // Get the native library name.
        final String nativeLibName = nativeLibInfo.getNativeLibName(libName);

        // Try to load the library file.
        try {
            System.load(new File(installDir, nativeLibName).getPath());
        } catch (UnsatisfiedLinkError ex) {
            // should be safe to continue as long as the native is loaded by any
            // loader
            if (ex.getMessage().indexOf("already loaded") == -1) {
                displayError("Unable to load " + nativeLibName);
                throw ex;
            }
        }
    }

    /**
     * The loadNativesAndStart method is the last step before launch :
     * System.load() the natives and init()/start() the child applet.
     * 
     * @param nativeLibDir The directory where the native libraries are stored.
     */
    private void loadNativesAndStart(final File nativeLibDir) {
        // back to the EDT
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                displayMessage("Loading native libraries");

                // disable JOGL loading from elsewhere
                com.sun.opengl.impl.NativeLibLoader.disableLoading();

                Class driHackClass = null;
                if (driHackPresent) {
                    // Load DRI hack library and run the DRI hack itself
                    loadLibrary(nativeLibDir, "jogl_drihack");
                    try {
                        driHackClass = Class
                                .forName("com.sun.opengl.impl.x11.DRIHack");
                        driHackClass.getMethod("begin", new Class[] {}).invoke(
                                null, new Object[] {});
                    } catch (Exception e) {
                        // TODO Add logger.
                        System.err.println(e);
                    }
                }

                // Load core JOGL native library
                loadLibrary(nativeLibDir, "jogl");

                if (driHackPresent) {
                    // End DRI hack
                    try {
                        driHackClass.getMethod("end", new Class[] {}).invoke(
                                null, new Object[] {});
                    } catch (Exception e) {
                        // TODO Add logger.
                        System.err.println(e);
                    }
                }

                if (!nativeLibInfo.isMacOS()) { // borrowed from NativeLibLoader
                    // Must pre-load JAWT on all non-Mac platforms to
                    // ensure references from jogl_awt shared object
                    // will succeed since JAWT shared object isn't in
                    // default library path
                    try {
                        System.loadLibrary("jawt");
                    } catch (UnsatisfiedLinkError ex) {
                        // Accessibility technologies load JAWT themselves; safe
                        // to
                        // continue
                        // as long as JAWT is loaded by any loader
                        if (ex.getMessage().indexOf("already loaded") == -1) {
                            displayError("Unable to load JAWT");
                            throw ex;
                        }
                    }
                }

                // Load AWT-specific native code
                loadLibrary(nativeLibDir, "jogl_awt");

                displayMessage("Starting DDSApplet");

                // start the subapplet
                startSubApplet();
            }
        });
    }

    /**
     * The refreshJOGL is executed from outside the Event Dispatch Thread, and
     * installs the required native libraries in the local folder.
     * 
     * @param installDir The installation direction.
     */
    private void refreshJOGL(final File installDir) {
        boolean result = installDir.exists() || installDir.mkdirs();

        String nativeJarName = null;
        URLConnection urlConnection = null;
        File localJarFile = null;

        if (result) {
            nativeJarName = nativeLibInfo.getNativeJarName();

            URL nativeLibURL = null;
            final String path = getCodeBase().toExternalForm() + nativeJarName;
            try {
                nativeLibURL = new URL(path);
                urlConnection = nativeLibURL.openConnection();
            } catch (MalformedURLException e) {
                System.err.println(e);
                // TODO Add logger here or in the displayError method.
                displayError("Malformed native lib URL : " + path);
                result = false;
            } catch (IOException e) {
                System.err.println(e);
                // TODO Add logger here or in the displayError method.
                displayError("Couldn't access the native lib URL : " + path);
                result = false;
            }
            displayMessage("Refreshing native libraries");
            // first download the full jar locally
            localJarFile = new File(installDir, nativeJarName);

            result = saveNativesJarLocally(localJarFile, urlConnection);
        } else {
            displayError("Unable to create directories for target: "
                    + installDir);
        }

        if (result) {
            try {
                final JarFile jarFile = new JarFile(localJarFile);

                // Iterate the entries finding all candidate libraries
                // that
                // need
                // to have their signatures verified
                if (findNativeEntries(jarFile)) {

                    final byte[] buf = new byte[EIGHT_KB];

                    // Now install the native library files
                    progressBar.setValue(0);
                    for (int i = 0; i < nativeLibNames.length; i++) {
                        displayMessage("Installing native files");
                        if (!installFile(installDir, jarFile,
                                nativeLibNames[i], buf)) {
                            return;
                        }
                        final int percent = HUNDRED * (i + 1)
                                / nativeLibNames.length;
                        progressBar.setValue(percent);
                    }

                    // At this point we can delete the jar file we just
                    // downloaded
                    jarFile.close();
                    localJarFile.delete();

                } else {
                    displayError("native libraries not found in jar file");
                    result = false;
                }

            } catch (IOException e) {
                System.err.println(e);
                // TODO Add logger here or in the displayError method.
                displayError("Error opening jar file " + localJarFile.getName()
                        + " for reading");
                result = false;
            }

            if (result) {
                // Still need to discover whether DRI hack is
                // installed...maybe?
                // TODO Determine if the DRI hack check is necessary.
                final File[] files = installDir.listFiles();
                for (int i = 0; i < files.length; i++) {
                    if (files[i].getName().indexOf("jogl_drihack") >= 0) {
                        driHackPresent = true;
                    }
                }

                loadNativesAndStart(installDir);
            }
        } else {
            // TODO Add logger here or in the displayError method.
            displayError("Unable to install the native file locally");
        }
    }

    /**
     * The saveNativesJarLocally method is responsible for storing the native
     * libraries on the host computer.
     * 
     * @param localJarFile A file descriptor for the jar file.
     * @param urlConnection The urlConnection?
     * @return true if the operation was successful.
     */
    // TODO Check comments for saveNativesJarLocally in DDSLauncher
    private boolean saveNativesJarLocally(final File localJarFile,
            final URLConnection urlConnection) {
        boolean result = false;
        BufferedOutputStream out = null;

        InputStream inStream = null;
        displayMessage("Downloading native library");
        progressBar.setValue(0);
        try {
            out = new BufferedOutputStream(new FileOutputStream(localJarFile));
            final int totalLength = urlConnection.getContentLength();
            inStream = urlConnection.getInputStream();
            final byte[] buffer = new byte[ONE_KB];
            int sum = 0;
            int len = inStream.read(buffer);
            while (len > 0) {
                out.write(buffer, 0, len);
                sum += len;
                final int percent = (HUNDRED * sum / totalLength);
                progressBar.setValue(percent);
                len = inStream.read(buffer);
            }
            result = true;
            out.close();
            inStream.close();
        } catch (FileNotFoundException e) {
            // TODO Add logger or something here.
            System.err.println(e);
        } catch (IOException e) {
            // TODO Add logger or something here.
            System.err.println(e);
        } finally {
            // close the files
            if (out != null) {
                try {
                    out.close();
                } catch (IOException ignore) {
                    // TODO Add logger or something here.
                    System.err.println(ignore);
                }
            }
            if (inStream != null) {
                try {
                    inStream.close();
                } catch (IOException ignore) {
                    // TODO Add logger or something here.
                    System.err.println(ignore);
                }
            }
        }
        return result;
    }

    /**
     * Associates the specified stream with the specified key in this applet
     * context. If the applet context previously contained a mapping for this
     * key, the old value is replaced.
     * <p>
     * For security reasons, mapping of streams and keys exists for each
     * codebase. In other words, applet from one codebase cannot access the
     * streams created by an applet from a different codebase
     * <p>
     * 
     * @param key key with which the specified value is to be associated.
     * @param stream stream to be associated with the specified key. If this
     *            parameter is null, the specified key is removed in this applet
     *            context.
     * @throws IOException if the stream size exceeds a certain size limit. Size
     *             limit is decided by the implementor of this interface.
     * @since JDK1.4
     * @see java.applet.AppletContext #setStream(java.lang.String,
     *      java.io.InputStream)
     */
    public final void setStream(final String key, final InputStream stream)
            throws IOException {
        // TODO Write setStream in DDSLauncher.
        new UnsupportedOperationException(
                "Need to write DDSLauncher.setStream()");
        System.out.println("setStream");
    }

    // This is the original refreshJOGL code. It's here just in case we need to
    // refer to it as the DDSLauncher gets developed further.
    // /**
    // * The refreshJOGL is executed from outside the Event Dispatch Thread, and
    // * installs the required native libraries in the local folder.
    // *
    // * @param installDir The installation direction.
    // */
    // private void refreshJOGL(final File installDir) {
    // if ((!installDir.exists()) && (!installDir.mkdirs())) {
    // displayError("Unable to create directories for target: "
    // + installDir);
    // return;
    // }
    //        
    // final String nativeJarName = nativeLibInfo.getNativeJarName();
    //        
    // URL nativeLibURL;
    // URLConnection urlConnection;
    // final String path = getCodeBase().toExternalForm() + nativeJarName;
    // try {
    // nativeLibURL = new URL(path);
    // urlConnection = nativeLibURL.openConnection();
    // } catch (Exception e) {
    // System.err.println(e);
    // // TODO Add logger here or in the displayError method.
    // displayError("Couldn't access the native lib URL : " + path);
    // return;
    // }
    //        
    // // the timestamp used to determine if we have to download the native jar
    // // again don't rely on the OS's timestamp to cache this
    // final long lastModified = getTimestamp(installDir, urlConnection
    // .getLastModified());
    // if (lastModified == urlConnection.getLastModified()) {
    // // Still need to discover whether DRI hack is installed
    // final File[] files = installDir.listFiles();
    // for (int i = 0; i < files.length; i++) {
    // if (files[i].getName().indexOf("jogl_drihack") >= 0) {
    // driHackPresent = true;
    // }
    // }
    // } else {
    // displayMessage("Updating local version of the native libraries");
    // // first download the full jar locally
    // final File localJarFile = new File(installDir, nativeJarName);
    // try {
    // saveNativesJarLocally(localJarFile, urlConnection);
    // } catch (IOException ioe) {
    // System.err.println(ioe);
    // // TODO Add logger here or in the displayError method.
    // displayError("Unable to install the native file locally");
    // return;
    // }
    //            
    // try {
    // final JarFile jarFile = new JarFile(localJarFile);
    //                
    // // Iterate the entries finding all candidate libraries that need
    // // to have their signatures verified
    // if (!findNativeEntries(jarFile)) {
    // displayError("native libraries not found in jar file");
    // return;
    // }
    //                
    // final byte[] buf = new byte[EIGHT_KB];
    //                
    // // Now install the native library files
    // progressBar.setValue(0);
    // for (int i = 0; i < nativeLibNames.length; i++) {
    // displayMessage("Installing native files");
    // if (!installFile(installDir, jarFile, nativeLibNames[i],
    // buf)) {
    // return;
    // }
    // final int percent = HUNDRED * (i + 1)
    // / nativeLibNames.length;
    // progressBar.setValue(percent);
    // }
    //                
    // // At this point we can delete the jar file we just downloaded
    // jarFile.close();
    // localJarFile.delete();
    //                
    // // If installation succeeded, write a timestamp for all of the
    // // files to be checked next time
    // try {
    // final File timestampFile = new File(installDir, "timestamp");
    // timestampFile.delete();
    // final BufferedWriter writer = new BufferedWriter(
    // new FileWriter(timestampFile));
    // writer.write("" + urlConnection.getLastModified());
    // writer.flush();
    // writer.close();
    // } catch (Exception e) {
    // System.err.println(e);
    // // TODO Add logger here or in the displayError method.
    // displayError("Error writing time stamp"
    // + " for native libraries");
    // return;
    // }
    //                
    // } catch (Exception e) {
    // System.err.println(e);
    // // TODO Add logger here or in the displayError method.
    // displayError("Error opening jar file " + localJarFile.getName()
    // + " for reading");
    // return;
    // }
    // }
    //        
    // loadNativesAndStart(installDir);
    // }

    // /**
    // * The getTimestamp.
    // *
    // * @param installDir The installation directory.
    // * @param timestamp The timestamp value.
    // * @return A timestamp value?
    // */
    // // TODO Comment this properly.
    // private long getTimestamp(final File installDir, final long timestamp) {
    // // Avoid returning valid value if timestamp file doesn't exist
    // try {
    // final BufferedReader reader = new BufferedReader(new FileReader(
    // new File(installDir, "timestamp")));
    // try {
    // final StreamTokenizer tokenizer = new StreamTokenizer(reader);
    // // Avoid screwing up by not being able to read full longs
    // tokenizer.resetSyntax();
    // tokenizer.wordChars('0', '9');
    // tokenizer.wordChars('-', '-');
    // tokenizer.nextToken();
    // final String tok = tokenizer.sval;
    // if (tok != null) {
    // return Long.parseLong(tok);
    // }
    // } catch (Exception e) {
    // // TODO Do something here, even if its just logging.
    // System.err.println(e);
    // } finally {
    // reader.close();
    // }
    // } catch (Exception e) {
    // // TODO Do something here, even if its just logging.
    // System.err.println(e);
    // }
    // return ((timestamp == 0) ? 1 : 0);
    // }

    /**
     * Replaces the Web page currently being viewed with the given URL. This
     * method may be ignored by applet contexts that are not browsers.
     * 
     * @param url an absolute URL giving the location of the document.
     * @see java.applet.AppletContext#showDocument(java.net.URL)
     */
    public final void showDocument(final URL url) {
        // TODO Write showDocument in DDSLauncher.
        new UnsupportedOperationException(
                "Need to write DDSLauncher.showDocument");
        System.out.println("showDocument(URL)");
    }

    /**
     * Requests that the browser or applet viewer show the Web page indicated by
     * the <code>url</code> argument. The <code>target</code> argument
     * indicates in which HTML frame the document is to be displayed. The target
     * argument is interpreted as follows:
     * <p>
     * <center><table border="3" summary="Target arguments and their
     * descriptions">
     * <tr>
     * <th>Target Argument</th>
     * <th>Description</th>
     * </tr>
     * <tr>
     * <td><code>"_self"</code>
     * <td>Show in the window and frame that contain the applet.</tr>
     * <tr>
     * <td><code>"_parent"</code>
     * <td>Show in the applet's parent frame. If the applet's frame has no
     * parent frame, acts the same as "_self".</tr>
     * <tr>
     * <td><code>"_top"</code>
     * <td>Show in the top-level frame of the applet's window. If the applet's
     * frame is the top-level frame, acts the same as "_self".</tr>
     * <tr>
     * <td><code>"_blank"</code>
     * <td>Show in a new, unnamed top-level window.</tr>
     * <tr>
     * <td><i>name</i>
     * <td>Show in the frame or window named <i>name</i>. If a target named
     * <i>name</i> does not already exist, a new top-level window with the
     * specified name is created, and the document is shown there.</tr>
     * </table> </center>
     * <p>
     * An applet viewer or browser is free to ignore <code>showDocument</code>.
     * 
     * @param url an absolute URL giving the location of the document.
     * @param target a <code>String</code> indicating where to display the
     *            page.
     * @see java.applet.AppletContext#showDocument(java.net.URL,
     *      java.lang.String)
     */
    public final void showDocument(final URL url, final String target) {
        // TODO Write showDocument in DDSLauncher.
        new UnsupportedOperationException(
                "Need to write DDSLauncher.showDocument");
        System.out.println("showDocument(URL, String)");
    }

    /**
     * Requests that the argument string be displayed in the "status window".
     * Many browsers and applet viewers provide such a window, where the
     * application can inform users of its current state.
     * 
     * @param status a string to display in the status window.
     * @see java.applet.Applet#showStatus(java.lang.String)
     */
    public final void showStatus(final String status) {
        // TODO Write showStatus in DDSLauncher.
        new UnsupportedOperationException(
                "Need to write DDSLauncher.showStatus");
        System.out.println("showStatus");
    }

    /**
     * The start method handles the asynchroneous loading of libraries if
     * needed.
     * 
     * @see java.applet.Applet#start()
     */
    public final void start() {
        // Check to make sure we initialized properly.
        // TODO Make sure this belongs here and not in the init method or
        // something.
        if (!isInitOk) {
            // TODO Add logger or complain about problematic startup?
            return;
        }

        // Check to see if this is the first time starting this.
        // TODO Determine if we really need to do this since we're no longer
        // running within a web browser.
        if (firstStart) {
            // Set the firstStart field to indicate that further calls to start
            // are not the first.
            firstStart = false;

            // Get the user's home directory.
            final String userHome = System.getProperty("user.home");

            // Pick the native library installation directory.
            final String installDirName = userHome + File.separator
                    + ".jogl_ext" + File.separator + installDirectory;

            // Create a File o bject the points to the install location.
            final File installDir = new File(installDirName);

            // // Create the installation directory if it doesn't exist to
            // prevent
            // if(!installDir.exists()) {
            // installDir.mkdirs();
            // }

            // Create a Thread that will allow loading of the native libraries
            // asynchronously with updating the display.
            final Thread refresher = new Thread() {
                public void run() {
                    // Make the call to refresh the local JOGL installation.
                    refreshJOGL(installDir);
                }
            };

            // Set the Thread priority below the rendering loop and start it.
            refresher.setPriority(Thread.NORM_PRIORITY - 1);
            refresher.start();
        } else if (joglStarted) { // Check to see if JOGL has already been
            // initialized.
            // Since it has we have to start again the applet (start can be
            // called
            // multiple times, e.g once per tabbed browsing.
            // TODO Determine if this is necessary since we're no longer running
            // this from a web browser.
            subApplet.start();
        }

        // Set the JFrame for the application to be visible.
        // TODO Make sure this is the appropriate place to do this.
        this.appletFrame.setVisible(true);
    }

    /**
     * The startSubApplet method is the true start of the sub applet (invoked in
     * the EDT).
     */
    private void startSubApplet() {
        // Create a new DDSApplet and set it to use this as a stub.
        subApplet = new DDSApplet();
        subApplet.setStub(this);

        // Add the DDSApplet to the center of the BorderLayout.
        add(subApplet, BorderLayout.CENTER);

        // init() the DDSApplet.
        subApplet.init();

        // Remove the loading panel.
        remove(loaderPanel);

        // Revalidate the components.
        // TODO Determine if we really need to do this.
        validate();

        // start() the DDSApplet.
        subApplet.start();

        // Set the flag that indicates that the JOGL is started.
        joglStarted = true;
    }

    /**
     * Called by the browser or applet viewer to inform this applet that it
     * should stop its execution. It is called when the Web page that contains
     * this applet has been replaced by another page, and also just before the
     * applet is to be destroyed.
     * <p>
     * A subclass of <code>Applet</code> should override this method if it has
     * any operation that it wants to perform each time the Web page containing
     * it is no longer visible. For example, an applet with animation might want
     * to use the <code>start</code> method to resume animation, and the
     * <code>stop</code> method to suspend the animation.
     * <p>
     * The implementation of this method provided by the <code>Applet</code>
     * class does nothing.
     * 
     * @see java.applet.Applet#destroy()
     * @see java.applet.Applet#init()
     * @see java.applet.Applet#stop()
     */
    public final void stop() {
        // Make sure we have a subApplet to stop.
        if (subApplet != null) {
            // Stop the subApplet.
            subApplet.stop();
        }
    }

}
