/**
 * ResourceType.java.
 */
package dds.control;

/**
 * The ResourceType enumeration is used to easily define information about the
 * different types of resources that the ResourceLoader and adata repository are
 * capable of handling. By implementing some extra functionality in this
 * enumeration we've been able to centralize the settings for diffrent resource
 * types. This means that we no longer have to worry about conflicting ints for
 * different types or complicated switch statements to retrieve directory and
 * other information about specific resource types.
 * 
 * @author Jacob Bolton
 */
public enum ResourceType {
    /**
     * The FONT is used to specify that a font is being requested.
     */
    FONT("fonts/"),

    /**
     * The IMAGE is used to specify that an image is being requested.
     */
    IMAGE("images/"),

    /**
     * The MAP is used to specify that a map is being requested.
     */
    MAP("maps/"),

    /**
     * The MODEL is used to specify that a model is being requested.
     */
    MODEL("models/"),

    /**
     * The SCENARIO is used to specify that a scenario is being requested.
     */
    SCENARIO("scenarios/"),

    /**
     * The SCRIPT is used to specify that a script is being requested.
     */
    SCRIPT("scripts/"), 
    
    /**
     * The DIRECT.
     */
    DIRECT("");

    /**
     * The directory field is a String representation of the path for this
     * particular ResourceType.
     */
    private final String directory;

    /**
     * Constructor for ResourceType.
     * 
     * @param dirName Location in the repository of this specific type of
     *            resource.
     */
    private ResourceType(final String dirName) {
        this.directory = dirName;
    }

    /**
     * Getter for directory.
     * 
     * @return the directory
     */
    public final String getDirectory() {
        return directory;
    }

}
