package dds.control.simulator.ai;

import java.net.URI;

import org.w3c.dom.Entity;

import dds.model.simulation.Vector3D;

/**
 * The AIFactory interface is used to enforce the requirement that the 
 * classes that implement this interface define the createAIManager()
 * method and the createAIScript() method. These methods are used by 
 * classes that interact with the concrete implementations of AIFactory
 * to create the AIManagers and AIScripts as needed.
 * 
 * @author jbolton
 * @author Mike Sanders
 */
public interface AIFactory {

    /**
     * Creates a new AIManager.
     * 
     * @param dom DOM entity
     * @return A new AIManager with associated scripts.
     */
    AIManager createAIManager(Entity dom);

    /**
     * The createAIScript method creates a new AIScript object based on 
     * the parameters passed in.
     * 
     * @param file Files to read AIScript from.
     * @param location Location to provide to script.
     * @param params Paranms to pass to script.
     * @return A new AIScript.
     */
    AIScript createAIScript(URI file, Vector3D location, AIParameter[] params);

} // end interface definition AIFactory
