package dds.control.simulator.ai.commands;

import bsh.CallStack;
import bsh.EvalError;
import bsh.Interpreter;
import dds.control.simulator.SimController;
import dds.model.simulation.Car;
import dds.model.simulation.Vector3D;

/**
 * The CreateSimObject class.
 * 
 * @author Mike Sanders
 */
// TODO Consider making this command take in a parameter for the SimObject type
// so it can add more than just cars.
// TODO Chain the usage of the different invoke commands to reduce code
// duplication.
public final class CreateSimObject {

    /**
     * The invoke method is called to execute this command.
     * 
     * @param env is the Interpreter environment that the command is executed
     *            in.
     * @param callstack is the CallStack of the environment.
     * @param xPos is the x position of the object.
     * @param yPos is the y position of the object.
     * @param zPos is the z position of the object.
     * @param xOrient is the x orientation of the object.
     * @param yOrient is the y orientation of the object.
     * @param zOrient is the z orientation of the object.
     */
    public static void invoke(final Interpreter env, final CallStack callstack,
            final float xPos, final float yPos, final float zPos,
            final float xOrient, final float yOrient, final float zOrient) {

        String targetName = "";

        try {
            targetName = (String) env.get("targetObject");
        } catch (EvalError e) {
            System.err.println(e.getMessage());
            System.err.println(e.getStackTrace());
        } // end try-catch

        final Car newCar = new Car(targetName);
        newCar.setPosition(xPos, yPos, zPos);
        newCar.setOrientation(xOrient, yOrient, zOrient);
        SimController.getSimControl().getSimulation().addObject(newCar);

    } // end method invoke()

    /**
     * The invoke method is called to execute this command.
     * 
     * @param env is the Interpreter environment that the command is executed
     *            in.
     * @param callstack is the CallStack of the environment.
     * @param objId is a String representing the idString of the new Car.
     * @param xPos is the x position of the object.
     * @param yPos is the y position of the object.
     * @param zPos is the z position of the object.
     * @param xOrient is the x orientation of the object.
     * @param yOrient is the y orientation of the object.
     * @param zOrient is the z orientation of the object.
     */
    public static void invoke(final Interpreter env, final CallStack callstack,
            final String objId, final float xPos, final float yPos,
            final float zPos, final float xOrient, final float yOrient,
            final float zOrient) {

        final Car newCar = new Car(objId);
        newCar.setPosition(xPos, yPos, zPos);
        newCar.setOrientation(xOrient, yOrient, zOrient);
        SimController.getSimControl().getSimulation().addObject(newCar);

    } // end method invoke()

    /**
     * The invoke method is called to execute this command.
     * 
     * @param env is the Interpreter environment that the command is executed
     *            in.
     * @param callstack is the CallStack of the environment.
     * @param objId is a String representing the idString of the new Car.
     * @param pos is the position of the object.
     * @param orient is the orientation of the object.
     */
    public static void invoke(final Interpreter env, final CallStack callstack,
            final String objId, final Vector3D pos, final Vector3D orient) {
        invoke(env, callstack, objId, pos.getXpos(), pos.getYpos(), pos
                .getZpos(), orient.getXpos(), orient.getYpos(), orient
                .getZpos());
    } // end method invoke()

    /**
     * Constructor for CreateSimObject.
     */
    private CreateSimObject() {
        super();
    }

} // end class definition CreateSimObject
