package dds.control.simulator.ai.commands;

import bsh.CallStack;
import bsh.EvalError;
import bsh.Interpreter;
import dds.control.simulator.SimCommand;
import dds.control.simulator.ai.AIScript;

/**
 * The CueSimCommand class defines the CueSimCommand AIScript command for use in
 * AIScript files. This command cues a new SimCommand with the AIScript so that
 * it executes at a certain time.
 * 
 * @author Mike Sanders
 */
public final class CueSimCommand {

    /**
     * Private constructor for CueSimCommand that prevents instantiation.
     */
    private CueSimCommand() {
        super();
    }

    /**
     * The invoke method is called to execute this command.
     * 
     * @param env is the Interpreter environment that the command is executed
     *            in.
     * @param callstack is the CallStack of the environment.
     * @param idString is a String representing the id of the SimCommand.
     * @param cmd is a String representing the method that needs to be executed.
     * @param args is an Object[] representing the arguments needed for command
     *            execution.
     * @param execTime is a float representing when the SimCommand should be
     *            executed.
     */
    public static void invoke(final Interpreter env, final CallStack callstack,
            final String idString, final String cmd, final Object[] args,
            final float execTime) {

        // get a new SimCommand from the pile...
        final SimCommand command = SimCommand.getCommand(idString, cmd, args,
                execTime);

        try {
            // get the execution environment...
            final AIScript script = (AIScript) env.get("script");
            // and cue the SimCommand for execution.
            script.cueCommand(command);
        } catch (EvalError e) {
            System.err.println(e.getScriptStackTrace());
        } // end try-catch

    } // end method invoke()

} // end class definition CueSimCommand

