/**
 * HasCuedCommands.java.
 */
package dds.control.simulator.ai.commands;

import dds.control.simulator.ai.AIScript;
import bsh.CallStack;
import bsh.EvalError;
import bsh.Interpreter;

/**
 * The HasCuedCommands class defines the HasCuedCommands script command for
 * AIScript files.
 * 
 * @author Jacob Bolton
 */
public final class HasCuedCommands {

    /**
     * Private constructor for HasCuedCommands to prevent insatntiation.
     */
    private HasCuedCommands() {
        super();
    }

    /**
     * The invoke method defines the parameters of the HasCuedCommands script
     * command. In the script, the HasCuedCommands command does not take in any
     * parameters. The result of the command will indicate if the current
     * AIScript has any SimCommands left in its queue of cued commands.
     * 
     * @param env The execution environment. - Not needed for script call.
     * @param callstack The call stack. - Not needed for script call.
     * @return True if there are more time-cued SimCommands left in the
     *         AIScript.
     */
    public static boolean invoke(final Interpreter env,
            final CallStack callstack) {
        // Set up a variable for the current AIScript.
        AIScript script = null;

        // Try to get and store a reference to the current AIScript.
        try {
            script = (AIScript) env.get("script");
        } catch (EvalError e) {
            // TODO Add logger
            System.err.println(e);
        }

        // Return the result of the AIScript's hasCuedCommands() method.
        return script.hasCuedCommands();
    }
}
