/**
 * TriggerOnce.java.
 */
package dds.control.simulator.ai.commands;

import bsh.CallStack;
import bsh.EvalError;
import bsh.Interpreter;

/**
 * The TriggerOnce class defines the TriggerOnce script command for AIScript
 * files.
 * 
 * @author Jacob Bolton
 */
public final class TriggerOnce {

    /**
     * Private constructor for TriggerOnce to prevent instantiation.
     */
    private TriggerOnce() {
        super();
    }

    /**
     * The invoke method defines a syntax for a TriggerOnce command.
     * 
     * @param env The execution environment. - Not needed for script call.
     * @param callstack The call stack. - Not needed for script call.
     * @param trigger A boolean trigger value.
     * @return Always return true after a single true has been passed in as a
     *         parameter.
     */
    public static boolean invoke(final Interpreter env,
            final CallStack callstack, final boolean trigger) {

        // Assume that our result will be false.
        boolean triggered = false;

        try {
            // See if we can get something useful from the script's triggerOnce
            // variable.
            if (env.get("triggerOnce") != null) {
                // There's data there, get it as a Boolean.
                triggered = (Boolean) env.get("triggerOnce");

                // If triggerOnce is false then overwrite with the passed in
                // value.
                if (!triggered) {
                    triggered = trigger;
                }
            }

            // Set the triggeredOnce variable to the current value of triggered.
            env.set("triggerOnce", triggered);
        } catch (EvalError e) {
            // TODO Add logger/
            System.out.println(e);
        }

        // Return triggered as the result.
        return triggered;
    }

}
