/**
 * MainMenuModule.java.
 */
package dds.control.user.modules;

import dds.control.DDSApplet;
import dds.control.user.ModuleType;

/**
 * The MainMenuModule class is responsible for presenting the user with a list
 * of options for program operation.
 * 
 * @author Jacob Bolton
 * @author krystian
 */
public class MainMenuModule extends AbstractListMenuModule {

    /**
     * Constructor for MainMenuModule.
     * 
     * @param applet The parenr DDSApplet.
     */
    public MainMenuModule(final DDSApplet applet) {
        super(applet);
        try {
            // Create options for the menu.
            ListMenuOption start = new ListMenuOption(this,
                    MainMenuModule.class.getMethod("startSimulation",
                            (Class[]) null), applet.loadImage("start.png"));
            ListMenuOption admin = new ListMenuOption(
                    this,
                    MainMenuModule.class.getMethod("adminMenu", (Class[]) null),
                    applet.loadImage("admin.png"));

            this.setBackground(applet.loadImage("menuback.jpg"));
            
            // Add the created options to the menu.
            this.addOption(start);
            
            // Only add administrative options if we're not running as an
            // applet.
            if (this.getParent().isStandAlone()) {
                this.addOption(admin);
            }
        } catch (SecurityException e) {
            // TODO Add logger.
            System.out.println(e);
        } catch (NoSuchMethodException e) {
            // TODO Add logger.
            System.out.println(e);
        }
    }

    /**
     * The startSimulation method defines the actions to take when the 'Start
     * Simulation' is pushed and a new simulation needs to be started.
     */
    public final void startSimulation() {
        this.getParent().loadModule(ModuleType.SIMULATION);
    }

    /**
     * The adminMenu method defines the actions to take when the
     * 'Administration' button is pushed and we need to switch into
     * administration mode.
     */
    public final void adminMenu() {
//        this.getParent().loadModule(ModuleType.ADMIN_MENU);
        this.getParent().loadModule(ModuleType.TRAP_EDITOR);
    }
}
