package dds.model.simulation;

/**
 * The Drivable interface is responsible for describing the standard methods
 * used to control a SimObject that can be driven, such as a Vehicle.
 * 
 * @author Jacob Bolton
 */
public interface Drivable {

    /**
     * The accelerate method causes the Driavable to maximally accelerate.
     */
    void accelerate();

    /**
     * The brake method causes the Driavable to maximally brake.
     */
    void brake();

    /**
     * The brake method applies a certain amount of braking to this Drivable.
     * 
     * @param amount Amount of braking to use 0.0 - 1.0
     */
    void brake(float amount);

    /**
     * The turn method causes the Drivable to change it's direction by a
     * specified amount.
     * 
     * @param amount Amount to turn. Values < 0 are for left turns; values > 0
     *            are for right turn.
     */
    void turn(float amount);

    /**
     * The accelerate method causes the Drivable to change it's forward
     * acceleraion by a specified amount.
     * 
     * @param amount The amount to accelerate by. This amount is additive, NOT
     *            multiplicative.
     */
    void accelerate(float amount);

    /**
     * The setTurn method sets the turning amount to a value rather than
     * changing it by a value.
     * 
     * @param amount The amount to set the turning to.
     */
    void setTurn(float amount);

    /**
     * The getTurn retrieves the current value for the turn amount.
     * 
     * @return A float describing the turning ammount. < 0 is left > 0 is right.
     */
    float getTurn();

    /**
     * The setAcceleration method setrs the acceleration amount to a value
     * rather than adjusting it by a value.
     * 
     * @param amount The amount to set the acceleration to.
     */
    void setAcceleration(float amount);

    /**
     * The getAcceleration retrieves the current acceleration amount.
     * 
     * @return A float with the current amount. < 0 for backwards, > 0 for
     *         forwards.
     */
    float getAcceleration();

    /**
     * The isAcceleration.
     * 
     * @return True if the Drivable is accelerating.
     */
    boolean isAccelerating();

    /**
     * The stopAcceleration method causes the drivable to cease all
     * acceleration.
     */
    void stopAcceleration();
}
