package dds.model.simulation;

import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;

import dds.control.DDSApplet;
import dds.control.user.modules.SimulationModule;

/**
 * The Simulation class.
 * 
 * @author Jacob Bolton
 * @author cyberdan
 * @author Mike Sanders
 */
public class Simulation implements ActionListener {

    // TODO Comment Simulation completely.

    /**
     * The THOUSAND.
     */
    private static final float THOUSAND = 1000.0f;

    /**
     * The BOUNDS_OFFSET.
     */
    private static final int BOUNDS_OFFSET = 5;

    /**
     * The MAX_COLLISIONS constant declares the total number of collisions
     * before trying to find a better spot?
     */
    private static final int MAX_COLLISIONS = 50;

    // Use the driver field instead, and if it's just location then store a
    // Vector3D, not
    // /**
    // * The driversLocationObject.
    // */
    // private AbstractSimObject driversLocationObject;

    // /**
    // * The hazardLocations.
    // */
    // // TODO Decide if we ae going to use or if we need this field.
    // private Collection<AbstractSimObject> hazardLocations;

    /**
     * The staticObjects.
     */
    private Collection<AbstractSimObject> staticObjects;

    // private AbstractDynamicObject dynamicObject;

    /**
     * The dynamicObjects.
     */
    private Collection<AbstractDynamicObject> dynamicObjects;

    /**
     * The driver.
     */
    private Car driver;

    /**
     * The timeLimit is the number of seconds the driver has to complete the
     * simulation path.
     */
    private transient int timeLimit;

    /**
     * The timesUp.
     */
    private transient boolean timesUp;

    /**
     * The xTerrainBounds.
     */
    private Point xTerrainBounds = null;

    /**
     * The yTerrainBounds.
     */
    private Point yTerrainBounds = null;

    /**
     * The setTerrainBoundaries.
     * 
     * @param xLower The lower x limit.
     * @param xUpper The upper x limit.
     * @param yLower The lower y limit.
     * @param yUpper The upper y limit.
     */
    public final void setTerrainBoundaries(final int xLower, final int xUpper,
            final int yLower, final int yUpper) {
        this.xTerrainBounds = new Point(xLower, xUpper);
        this.yTerrainBounds = new Point(yLower, yUpper);
    }

    /**
     * The getXMapBounds.
     * 
     * @return The xMapBounds.
     */
    public final Point getXMapBounds() {
        return xTerrainBounds;
    }

    /**
     * The getYMapBounds.
     * 
     * @return The yMapBounds.
     */
    public final Point getYMapBounds() {
        return yTerrainBounds;
    }

    /**
     * Getter for dynamicObjects.
     * 
     * @return the dynamicObjects
     */
    public final Collection<AbstractDynamicObject> getDynamicObjects() {
        return dynamicObjects;
    }

    /**
     * Setter for dynamicObjects.
     * 
     * @param objs the dynamicObjects to set
     */
    public final void setDynamicObjects(
            final Collection<AbstractDynamicObject> objs) {
        this.dynamicObjects = objs;
    }

    /**
     * Getter for staticObjects.
     * 
     * @return the staticObjects
     */
    public final Collection<AbstractSimObject> getStaticObjects() {
        return staticObjects;
    }

    /**
     * Setter for staticObjects.
     * 
     * @param objects the staticObjects to set
     */
    public final void setStaticObjects(
            final Collection<AbstractSimObject> objects) {
        this.staticObjects = objects;
    }

    /**
     * The simulate method is the method called by the main Simulation driver.
     * This method is responsible for updating the Kinetics of the dynamic
     * objects and then checking for collisions. If any collision has been
     * detected with a driver, then it calls the handleCollision functions.
     * 
     * @param time Time change to simulate.
     */
    public final void simulate(final float time) {
        // TODO Should lastPosition be the same object as the current one?
        final Vector3D lastPosition = this.driver.getPosition().duplicate();
        // final Vector3D lastPosition = this.driver.getPosition().duplicate();
        AbstractSimObject collidedWith = null;
        boolean collided = false;
        int collisionCounter = 0;

        // TODO Find out why we divide by 1000 here.
        this.driver.simulate(time / THOUSAND);

        for (AbstractDynamicObject simObject : this.dynamicObjects) {
            if (this.driver.detectCollision(simObject)) {
                collidedWith = simObject;
                collided = true;
                // TODO Determine if we should deal with ALL dynamic object
                // collisions instead of only one.
                break;
            }
        }

        if (!collided) {
            for (AbstractSimObject staticObj : this.staticObjects) {
                if (this.driver.detectCollision(staticObj)) {
                    collidedWith = staticObj;
                    collided = true;
                    // TODO Determine if we should deal with ALL dynamic object
                    // collisions instead of only one.
                    break;
                }
            }
        }

        while (collided && collisionCounter < MAX_COLLISIONS) {
            collisionCounter++;

            this.driver.handleCollision(collidedWith, lastPosition);

            if ((collidedWith instanceof Road)
                    || (collidedWith instanceof Intersection)) {
                // TBD...
                collided = false;
            } else {
                // this.driver.handleCollision(collidedWith, lastPosition);
                collided = this.driver.detectCollision(collidedWith);
            }
        }

        this.enforceMapBounds();
    } // end of method simulate

    /**
     * The enforceMapBounds.
     */
    private void enforceMapBounds() {
        boolean changed = false;
        float driverX = this.driver.getPosition().getXpos();
        float driverY = this.driver.getPosition().getYpos();
        final float driverZ = this.driver.getPosition().getZpos();
//        final float oldDriverX = driverX;
//        final float oldDriverY = driverY;

        if (driverX < this.xTerrainBounds.x + BOUNDS_OFFSET) {
            driverX = this.xTerrainBounds.x + BOUNDS_OFFSET;
            changed = true;
        }
        if (driverX > this.xTerrainBounds.y - BOUNDS_OFFSET) {
            driverX = this.xTerrainBounds.y - BOUNDS_OFFSET;
            changed = true;
        }
        if (driverY < this.yTerrainBounds.x + BOUNDS_OFFSET) {
            driverY = this.yTerrainBounds.x + BOUNDS_OFFSET;
            changed = true;
        }
        if (driverY > this.yTerrainBounds.y - BOUNDS_OFFSET) {
            driverY = this.yTerrainBounds.y - BOUNDS_OFFSET;
            changed = true;
        }

        if (changed) {
            this.driver.setPosition(driverX, driverY, driverZ);
            // // TODO Move this polyloop stuff into the setPosition of
            // SimObjects
            // if (this.driver.getPolyLoop() != null) {
            // this.driver.getPolyLoop().moveMe(driverX - oldDriverX,
            // driverY - oldDriverY);
            // }
            // System.out.println("Trying to reposition driver to: (" + driverX
            // + ", " + driverY + "," + driverZ + ")");
        }
        // TODO Check driver's coordinates against map boundaries and move them
        // to within if necessary.

    }

    /**
     * The isTerminated.
     * 
     * @return True if the simulation is completed (i.e. map complete or
     *         collision detected)
     */
    public final boolean isTerminated() {
        // TODO Write isTerminated
        return false;
    }

    /**
     * The destruct.
     */
    public final void destruct() {
        this.staticObjects.clear();
        this.dynamicObjects.clear();
    }

    /**
     * The getObject.
     * 
     * @param idStr The identifying String of an object to retrieve.
     * @return The object if it exists or null if it doesn't.
     */
    public final AbstractSimObject getObject(final String idStr) {
        int flagFound = -1;
        AbstractSimObject toReturn = null;

        if (idStr.equals(SimulationModule.DRIVER)) {
            toReturn = this.driver;
        }

        if (flagFound != 1) {
            for (AbstractSimObject simObject : this.staticObjects) {
                if (simObject.getIdStr().equals(idStr)) {
                    flagFound = 1;
                    toReturn = simObject;
                }
            }
        }

        if (flagFound != 1) {
            for (AbstractSimObject simObject : this.dynamicObjects) {
                if (simObject.getIdStr().equals(idStr)) {
                    flagFound = 1;
                    toReturn = simObject;
                }
            }
        }

        return toReturn;
    }

    /**
     * add an object to the appropriate list.
     * 
     * @param aso AbstractSimObject to add
     */
    public final void addObject(final AbstractSimObject aso) {
        int flagAdded = -1;
        // is it an dynamic object?
        if (aso instanceof AbstractDynamicObject) {
            this.dynamicObjects.add((AbstractDynamicObject) aso);
            flagAdded = 1;
        }

        // no, then do this
        if ((flagAdded != 1) && (aso instanceof AbstractSimObject)) {
            this.staticObjects.add(aso);
        }
    }

    /**
     * The removeDynamicObject method is used to remove dynamic objects from the
     * simulation.
     * 
     * @param idString is a String representing the idString of the
     *            AbstractDynamicObject to be removed from the simulation.
     */
    public final void removeDynamicObject(final String idString) {

        for (AbstractDynamicObject toRemove : dynamicObjects) {
            if (toRemove.getIdStr().equals(idString)) {
                dynamicObjects.remove(toRemove);
                ((SimulationModule) DDSApplet.getApplet().getModuleLoader()
                        .getCurrentModule()).removeMovingObject(toRemove);
                break;
            } // end if
        } // end for-each

    } // end method removeDynamicObject()

    /**
     * Constructor for Simulation.
     */
    public Simulation() {
        super();
        this.staticObjects = new ArrayList<AbstractSimObject>();
        this.dynamicObjects = new ArrayList<AbstractDynamicObject>();
    }

    /**
     * Constructor for Simulation.
     * 
     * @param sobjs Collection of static SimObjects.
     * @param dobjs Collection of dynamic Dynamic objects.
     */
    public Simulation(final Collection<AbstractSimObject> sobjs,
            final Collection<AbstractDynamicObject> dobjs) {
        this();
        staticObjects.addAll(sobjs);
        dynamicObjects.addAll(dobjs);
    }

    /**
     * Getter for driver.
     * 
     * @return the driver
     */
    public final Car getDriver() {
        return driver;
    }

    /**
     * Setter for driver.
     * 
     * @param newDriver the driver to set
     */
    public final void setDriver(final Car newDriver) {
        this.driver = newDriver;
    }

    /**
     * Getter for xTerrainBounds.
     * 
     * @return the xTerrainBounds
     */
    public final Point getXTerrainBounds() {
        return xTerrainBounds;
    }

    /**
     * Setter for xTerrainBounds.
     * 
     * @param terrainBounds the xTerrainBounds to set
     */
    public final void setXTerrainBounds(final Point terrainBounds) {
        xTerrainBounds = terrainBounds;
    }

    /**
     * Getter for yTerrainBounds.
     * 
     * @return the yTerrainBounds
     */
    public final Point getYTerrainBounds() {
        return yTerrainBounds;
    }

    /**
     * Setter for yTerrainBounds.
     * 
     * @param terrainBounds the yTerrainBounds to set
     */
    public final void setYTerrainBounds(final Point terrainBounds) {
        yTerrainBounds = terrainBounds;

    }

    /**
     * The setTimeLimit.
     * 
     * @param time specifies the amount of time the driver has to complete the
     *            path.
     */
    public final void setTimeLimit(final float time) {
        timeLimit = (int) time;
        timesUp = false;
        // timer = new Timer((int) THOUSAND, this);
    } // end method setTimeLimit()

    /**
     * The actionPerformed.
     * 
     * @param evt is the ActionEvent
     * @see java.awt.event.ActionListener
     *      #actionPerformed(java.awt.event.ActionEvent)
     */
    public final void actionPerformed(final ActionEvent evt) {
        if (timeLimit == 0) {
            timesUp = true;
        } else {
            --timeLimit;
        } // end if-else
    } // end method actionPerformed()

    /**
     * The isTimesUp.
     * 
     * @return timesUp
     */
    public final boolean isTimesUp() {
        return timesUp;
    }

    /**
     * The toString method creates a string representation of this object.
     * 
     * @return The String representing this object.
     * @see java.lang.Object#toString()
     */
    @Override
    public final String toString() {
        // TODO Consider a less verbose output.
        String str = "";
        for (AbstractSimObject obj : this.staticObjects) {
            str = str + obj.toString() + "\n";
        }
        for (AbstractSimObject obj : this.dynamicObjects) {
            str = str + obj.toString() + "\n";
        }
        return str;
    }

    /**
     * The calculateAll method causes all SimObjects within this simulation to
     * have all of their calculations performed.
     */
    public final void calculateAll() {
        // Do calculations for the driver's vehicle.
        this.driver.calculateAll();

        // Do calculations for the static objects.
        for (AbstractSimObject obj : this.staticObjects) {
            obj.calculateAll();
        }

        // Do calculations for the dynamic objects.
        for (AbstractSimObject obj : this.dynamicObjects) {
            obj.calculateAll();
        }
    }

} // end class definition Simulation
