package dds.model.trapeditor;

import processing.core.PGraphics;

/**
 * The INode2D class represents an intersection node in 2D.
 * 
 * @author cyberdan
 */
public class INode2D extends TNode2D {

    /**
     * Constructor for INode2D. Simply initialize the coordiante arrays.
     */
    public INode2D() {
        // TODO Make sure there's a super() defined in TNode2D
        super();
        this.setNpoints(1);
        this.setXcors(new float[1]);
        this.setYcors(new float[1]);
    }

    /**
     * The toString.
     * 
     * @return A String representation of this object.
     * @see java.lang.Object#toString()
     */
    public final String toString() {
        return ("INode2D: " + this.getName() + ": " + super.toString());
    }

    /**
     * The inBounds method determines if the point (x, y) is contained in the
     * region defined by this TNode2D.
     * 
     * @param xCoord the x-coordinate
     * @param yCoord the y-coordinate
     * @return true if in bounds; false otherwise
     * @see dds.control.user.modules.TNode2D#inBounds(int, int)
     */
    public final boolean inBounds(final int xCoord, final int yCoord) {
        return ((Math.pow(xCoord - getXcors()[0], 2) + Math.pow(yCoord
                - getYcors()[0], 2)) <= 100);
    }

    /**
     * The drawNode method draws the child object's node. A node can represent
     * an intersection, road, or block.
     * 
     * @param context the graphics context
     * @see dds.control.user.modules.TNode2D#drawNode()
     */
    @Override
    public final void drawNode(final PGraphics context) {
        context.fill(0xff0000e6);
        context.ellipse(getXcors()[0], getYcors()[0], 10, 10);
    }
}
