package dds.model.trapeditor;

import processing.core.PGraphics;

/**
 * The TNode2D class represents a 2-dimensional map tree node that is to be used
 * in the trap editor module.
 * 
 * @author cyberdan
 */
// TODO Fully comment and javadoc.
// TODO Rename class to AbstractTNode2D as per naming conventions.
public abstract class TNode2D {

    /**
     * The npoints is the number of vertices used to draw this node.
     */
    private int npoints = 0;

    /**
     * The xcors is the list of x coordinates.
     */
    private float[] xcors = null;

    /**
     * The ycors is the list of y coordinates.
     */
    private float[] ycors = null;

    /**
     * The xyMapper is the boundary mapper for this node. This should be the
     * same per map.
     */
    private TrapEditorXYMapper xyMapper = null;

    /**
     * The name, or identification string, for this node's representation as in
     * the simulation.
     */
    private String name = null;

    /**
     * The getName is the accessor for 'name'.
     * 
     * @return the name of this
     */
    public final String getName() {
        return name;
    }

    /**
     * The setName is the modifier for 'name'.
     * 
     * @param nameStr the new name for this
     */
    public final void setName(final String nameStr) {
        this.name = nameStr;
    }

    /**
     * The addPoint method adds a vertex to this node.
     * 
     * @param iindex the index into the xcors & ycors array
     * @param xCoord the x coordinate
     * @param yCoord the y coordinate
     */
    public final void addPoint(final int iindex, final float xCoord,
            final float yCoord) {
        this.xcors[iindex] = xCoord;
        this.ycors[iindex] = yCoord;
    }

    /**
     * The remapPoints does a change of coordinate system for all the vertices
     * stored in this node.
     */
    public final void remapPoints() {
        for (int p = 0; p < npoints; p++) {
            final float[] nextCor = new float[] { xcors[p], ycors[p] };
            xyMapper.getMappedCoordinate(nextCor);
            xcors[p] = nextCor[0];
            ycors[p] = nextCor[1];
        }
    }

    /**
     * The toString.
     * 
     * @return A String representation of this object.
     * @see java.lang.Object#toString()
     */
    public String toString() {
        String ret = "{";
        int index = 0;
        for (; index < npoints - 1; index++) {
            ret = ret + "(" + xcors[index] + ", " + ycors[index] + "), ";
        }
        if (npoints > 0) {
            ret = ret + "(" + xcors[index] + ", " + ycors[index] + ")";
        }
        ret = ret + "}";
        return ret;
    }

    /**
     * The getNpoints is the accessor for npoints.
     * 
     * @return the npoints
     */
    public final int getNpoints() {
        return npoints;
    }

    /**
     * The setNpoints is the modifier for npoints.
     * 
     * @param points the new npoints
     */
    public final void setNpoints(final int points) {
        this.npoints = points;
    }

    /**
     * The getXcors is the accessor for xcors.
     * 
     * @return the xcors
     */
    public final float[] getXcors() {
        return xcors;
    }

    /**
     * The setXcors is the modifier for xcors.
     * 
     * @param cors the new xcors
     */
    public final void setXcors(final float[] cors) {
        this.xcors = cors;
    }

    /**
     * The getXyMapper returns the xyMapper for this object.
     * 
     * @return the xyMapper
     */
    public final TrapEditorXYMapper getXyMapper() {
        return xyMapper;
    }

    /**
     * The setXyMapper is the modifier for xyMapper.
     * 
     * @param mapper the new xyMapper
     */
    public final void setXyMapper(final TrapEditorXYMapper mapper) {
        this.xyMapper = mapper;
    }

    /**
     * The getYcors is the accessor for ycors.
     * 
     * @return the ycors
     */
    public final float[] getYcors() {
        return ycors;
    }

    /**
     * The setYcors is the modifier for ycors.
     * 
     * @param cors the new ycors
     */
    public final void setYcors(final float[] cors) {
        this.ycors = cors;
    }

    /**
     * The inBounds method determines if the point (x, y) is contained in the
     * region defined by this TNode2D.
     * 
     * @param xCoord the x-coordinate
     * @param yCoord the y-coordinate
     * @return true if in bounds; false otherwise
     */
    public abstract boolean inBounds(int xCoord, int yCoord);

    /**
     * The drawNode method draws the child object's node. A node can represent
     * an intersection, road, or block.
     * 
     * @param context the graphics context
     */
    public abstract void drawNode(PGraphics context);
}
