package dds.model.trapeditor;

import java.awt.Rectangle;

/**
 * The TrapEditorXYMapper class is responsible for mapping x and y 
 * coordinates from one coordinate system to another.  The mapped 
 * coordinates can be in global or local form.
 * 
 * @author cyberdan
 */
public class TrapEditorXYMapper {

    /**
     * The srcBoundary is the source coordinate system.
     */
    private Rectangle srcBoundary = null;

    /**
     * The dstBoundary is the target coordinate system.
     */
    private Rectangle dstBoundary = null;

    /**
     * Constructor for TrapEditorXYMapper that takes in 2 boundaries 
     * that define the to-be-mapped coordinate systems.
     * 
     * @param srcBoundary
     * @param dstBoundary
     */
    public TrapEditorXYMapper(Rectangle srcBoundary, Rectangle dstBoundary) {
        this.srcBoundary = srcBoundary;
        this.dstBoundary = dstBoundary;
    }

    /**
     * The getDstBoundary.
     * @return
     */
    public Rectangle getDstBoundary() {
        return dstBoundary;
    }

    /**
     * The setDstBoundary.
     * @param dstBoundary
     */
    public void setDstBoundary(Rectangle dstBoundary) {
        this.dstBoundary = dstBoundary;
    }

    /**
     * The getSrcBoundary.
     * @return
     */
    public Rectangle getSrcBoundary() {
        return srcBoundary;
    }

    /**
     * The setSrcBoundary.
     * @param srcBoundary
     */
    public void setSrcBoundary(Rectangle srcBoundary) {
        this.srcBoundary = srcBoundary;
    }

    /**
     * The getMappedCoordinate method finds the x, y mapped 
     * coordinate with respect to the destination boundary.
     * Warning: this method modifies the parameter.
     * 
     * @param srcXY the coordinate to map
     */
    public void getMappedCoordinate(float[] srcXY) {
        final float x = srcXY[0];
        final float y = srcXY[1];

        srcXY[0] = ((((float) (x - srcBoundary.x)) / (float) srcBoundary.width) * (float) dstBoundary.width);
        srcXY[1] = ((((float) (y - srcBoundary.y)) / (float) srcBoundary.height) * (float) dstBoundary.height);
    }

    /**
     * The getUnmappedCoordinate method does the inverse operation of
     * getMappedCoordinate().
     * 
     * @param dstXY the coordiante to unmap
     */
    public void getUnmappedCoordinate(float[] destXY) {
        final float xp = destXY[0];
        final float yp = destXY[1];

        destXY[0] = ((((float) xp) / (float) dstBoundary.width) * (float) srcBoundary.width)
                + (float) srcBoundary.x;
        destXY[1] = ((((float) yp) / (float) dstBoundary.height) * (float) srcBoundary.height)
                + (float) srcBoundary.y;
    }
}
