package dds.view;

import dds.control.DDSApplet;

/**
 * The RenderableContext interface defines the method which the DDSApplet uses
 * to render each frame of the application. When a RenderableContext is set in
 * the DDSApplet the drawing function (as defined in Processing) gets passed
 * directly to the RenderableContext, which is responsible for performing the
 * appropriate drawing actions depending on the context in which they are
 * received. Currently the DDSApplet has it's displaying and input handling
 * functionality in separate classes, which can make the processing of input
 * that affect the display somewhat tedious. Since BOTH InputHandler and
 * RenderableContext are an interfaces, there's no reason why a singular class
 * can't implement both and be singular modules for DDSApplet operation.
 * 
 * @author Jacob Bolton
 */
public interface RenderableContext {

    /**
     * The drawOn method defines the appropriate drawing functions that are
     * executed to render each frame of the application.
     * 
     * @param applet The applet on which to draw the context.
     */
    void drawOn(DDSApplet applet);
}
