package dds.view.simplegui;

import java.awt.Rectangle;
import java.util.ArrayList;

import dds.control.DDSApplet;

/**
 * The SimpleCanvas class.
 * 
 * @author cyberdan
 */
public class SimpleCanvas extends SimpleAbstractGUIElement {

    /**
     * The imageContainers are various placeholders for images on this canvas.
     */
    private ArrayList<SimpleImageContainer> imageContainers = null;

    /**
     * The numOfContainers is the number of containers in this canvas.
     */
    private int numOfContainers = 0;

    /**
     * Default constructor for SimpleCanvas.
     */
    public SimpleCanvas() {
        this(new Rectangle(0, 0, DDSApplet.getApplet().width, DDSApplet
                .getApplet().height));
    }

    /**
     * Constructor for SimpleCanvas that takes in a boundary spec.
     * 
     * @param boundary the boundary to set for this component
     */
    public SimpleCanvas(final Rectangle boundary) {
        super();
        this.setBoundary(boundary);
        this.imageContainers = new ArrayList<SimpleImageContainer>();
    }

    /**
     * The addImageContainer method adds an image container.
     * 
     * @param sic the image container to add
     */
    public final void addImageContainer(final SimpleImageContainer sic) {
        this.imageContainers.add(sic);
        ++this.numOfContainers;
    }

    /**
     * The dispose method tries to free up any book-keeping info. for this
     * subclass.
     * 
     * @see dds.control.user.modules.SimpleGUI.SimpleAbstractGUIElement
     *      #dispose()
     */
    @Override
    public void dispose() {
        // TODO Write dispose in SimpleCanvas.
    }

    /**
     * The draw method draws this component to the specified applet.
     * 
     * @param app the drawing screen
     * @see dds.control.user.modules.SimpleGUI.SimpleAbstractGUIElement
     *      #draw(dds.control.DDSApplet)
     */
    @Override
    public final void draw(final DDSApplet app) {
        app.stroke(0xffc80000);
        app.strokeWeight(3.5f);

        app.rect(this.getBoundary().x, this.getBoundary().y,
                this.getBoundary().width, this.getBoundary().height);
        for (SimpleImageContainer imgContainer : this.imageContainers) {
            imgContainer.draw(app);
        }
    }

    /**
     * The setup method setups up any book-keeping info. on a subclass.
     * 
     * @see dds.control.user.modules.SimpleGUI.SimpleAbstractGUIElement#setup()
     */
    @Override
    public final void setup() {
        // TODO Write setup in SimpleCanvas.
    }

    /**
     * The handleMouseClick handles a mouse clicked event for this component.
     * 
     * @param mouseX mouseX
     * @param mouseY mouseY
     * @param mouseButton mouseButton
     * @see dds.control.user.modules.SimpleGUI.SimpleAbstractGUIElement
     *      #handleMouseClick(int, int, int)
     */
    public final void handleMouseClick(final int mouseX, final int mouseY,
            final int mouseButton) {
        // TODO Write handleMouseClick in SimpleCanvas.
    }
}
