package dds.view.simplegui;

import java.awt.Rectangle;
import java.awt.image.BufferedImage;

import processing.core.PImage;
import dds.control.DDSApplet;

/**
 * The SimpleImageContainer class.
 * @author cyberdan
 */
public class SimpleImageContainer extends SimpleAbstractGUIElement {

    /**
     * The image contained in this object.
     */
    private PImage image = null;

    /**
     * Default constructor for SimpleImageContainer: sets the current
     * image to the parameter 'image'.
     * @param image the image to set it
     */
    public SimpleImageContainer(PImage image) {
        this.image = image;
        this.setBoundary(new Rectangle(0, 0, this.image.width,
                this.image.height));
    }

    /**
     * Constructor for SimpleImageContainer that takes in a path name.
     * @param imgPath the image file name
     */
    public SimpleImageContainer(String imgPath) {
        this.image = DDSApplet.getApplet().loadImage(imgPath);
        this.setBoundary(new Rectangle(0, 0, this.image.width,
                this.image.height));
    }

    /**
     * Constructor for SimpleImageContainer that takes in an image 
     * and sets the boundary as given.
     * @param img the contained image
     * @param boundary the boundary of this component
     */
    public SimpleImageContainer(PImage img, Rectangle boundary) {
        this.image = img;
        this.setBoundary(boundary);
    }

    /**
     * The getImage accesses the image attribute.
     * @return
     */
    public PImage getImage() {
        return (this.image);
    }

    /**
     * The setImage is the modifier for the image attribute.
     * @param img
     */
    public void setImage(PImage img) {
        this.image = img;
    }

    /**
     * The setImage sets the image by the pathName.
     * @param pathName the file name of the image
     */
    public void setImage(String pathName) {
        this.setImage(DDSApplet.getApplet().loadImage(pathName));
    }

    public void setImage(BufferedImage img) {
    }

    @Override
    public void dispose() {
        // TODO Auto-generated method stub

    }

    @Override
    public void draw(DDSApplet app) {
        app.stroke(255, 255, 255);
        app.strokeWeight(1.25f);
        app.rect(this.getBoundary().x, this.getBoundary().y,
                this.getBoundary().width, this.getBoundary().height);
        app.image(this.image, this.getBoundary().x, this.getBoundary().y);
    }

    @Override
    public void handleMouseClick(int mouseX, int mouseY, int mouseButton) {
        // TODO Auto-generated method stub

    }

    @Override
    public void setup() {
        // TODO Auto-generated method stub

    }

}
