package dds.view.simplegui;

import java.awt.Rectangle;

import dds.control.DDSApplet;

import processing.core.PFont;

/**
 * The SimpleTextArea class.
 * 
 * @author cyberdan
 */
public class SimpleTextArea extends SimpleAbstractGUIElement {
    /**
     * The text is what is written in the text area.
     */
    private String text;

    /**
     * The font of the text.
     */
    private transient PFont font;

    /**
     * Default constructor for SimpleTextArea. Just chains to other constructors
     * with initial data.
     */
    public SimpleTextArea() {
        this("text area");
    }

    /**
     * Constructor for SimpleTextArea with an initial text spec.
     * 
     * @param initialText use this text as the default
     */
    public SimpleTextArea(final String initialText) {
        this(initialText, "FFScala", 32);
    }

    /**
     * Constructor for SimpleTextArea with an initial text spec as well as font
     * type.
     * 
     * @param initialText the initial text
     * @param fontName the font to use
     * @param fontSize the font size to use
     */
    public SimpleTextArea(final String initialText, final String fontName,
            final int fontSize) {
        this(initialText, fontName, fontSize, new Rectangle(0, 0, DDSApplet
                .getApplet().width, DDSApplet.getApplet().height));
    }

    /**
     * Constructor for SimpleTextArea with an initial text spec as well as font
     * type and a boundary.
     * 
     * @param initialText the initial text
     * @param fontName the font to use
     * @param fontSize the font size to use
     * @param bounds the bounds to use
     */
    public SimpleTextArea(final String initialText, final String fontName,
            final int fontSize, final Rectangle bounds) {
        super();
        this.text = initialText;
        this.font = DDSApplet.getApplet().createFont(fontName, fontSize);
        this.setBoundary(bounds);
    }

    /**
     * The getText method is the accessor for the text field.
     * 
     * @return The text field.
     */
    public final String getText() {
        return text;
    }

    /**
     * The setText method is the setter for the text field..
     * 
     * @param textStr The new value for the text field.
     */
    public final void setText(final String textStr) {
        this.text = textStr;
    }

    /**
     * The setup method setups up any book-keeping info. on a subclass.
     * 
     * @see dds.control.user.modules.SimpleGUI.SimpleAbstractGUIElement#setup()
     */
    public final void setup() {
        // TODO Write setup.
    }

    /**
     * The dispose method tries to free up any book-keeping info. for this
     * subclass.
     * 
     * @see dds.control.user.modules.SimpleGUI.SimpleAbstractGUIElement
     *      #dispose()
     */
    public final void dispose() {
        // TODO Write dispose.
    }

    /**
     * The draw method draws this component to the specified applet.
     * 
     * @param app the drawing screen
     * @see dds.control.user.modules.SimpleGUI.SimpleAbstractGUIElement
     *      #draw(dds.control.DDSApplet)
     */
    public final void draw(final DDSApplet app) {
        app.textFont(font);
        app.fill(0xfffa0064);
        app.text(this.text, (float) this.getBoundary().x, (float) this
                .getBoundary().y);
    }

    /**
     * The handleMouseClick handles a mouse clicked event for this component.
     * 
     * @param mouseX mouseX
     * @param mouseY mouseY
     * @param mouseButton mouseButton
     * @see dds.control.user.modules.SimpleGUI.SimpleAbstractGUIElement
     *      #handleMouseClick(int, int, int)
     */
    public void handleMouseClick(final int mouseX, final int mouseY,
            final int mouseButton) {
        // TODO Write handleMouseClick.
    }
}
