package dds.view.simulation;

import dds.model.simulation.AbstractDynamicObject;

/**
 * The AbstractDynamicObjectRenderer class is responsible for drawing a
 * DynamicObject from the Simulation onto the screen. DynamicObjects have some
 * sort of state data, which may affect how they should be displayed so
 * implementations of this renderer will need to keep track of that information
 * through the viewState object. The viewState object is simply an Object that
 * will appropriately represent all of the state information. Each
 * implementation will handle the interpretation and storage individually.
 * 
 * @author Jacob Bolton
 */
public abstract class AbstractDynamicObjectRenderer extends
        AbstractSimObjectRenderer {

    /**
     * Getter for viewState.
     * 
     * @return the viewState
     */
    public abstract Object getViewState();

    /**
     * Setter for viewState.
     * 
     * @param stateobj the viewState to set
     */
    public abstract void setViewState(final Object stateobj);

    /**
     * Constructor for AbstractDynamicObjectRenderer.
     * 
     * @param obj The AbstractDynamicObject that this class will render.
     */
    public AbstractDynamicObjectRenderer(final AbstractDynamicObject obj) {
        super(obj);
        this.setViewState(obj.getState());
    }

}
