package dds.view.simulation;

import dds.control.DDSApplet;
import dds.model.simulation.AbstractMovableObject;
import dds.model.simulation.Kinetics;

/**
 * The AbstractMovableObjectRenderer class.
 * 
 * @author Jacob Bolton
 */
public abstract class AbstractMovableObjectRenderer extends
        AbstractDynamicObjectRenderer {

    /**
     * The viewKinetics.
     */
    private Kinetics viewKinetics;

    /**
     * Getter for viewKinetics.
     * 
     * @return the viewKinetics
     */
    public final Kinetics getViewKinetics() {
        return viewKinetics;
    }

    /**
     * Setter for viewKinetics.
     * 
     * @param kinetics the viewKinetics to set
     */
    public final void setViewKinetics(final Kinetics kinetics) {
        this.viewKinetics = kinetics;
    }

    /**
     * The drawOn.
     * 
     * @param applet The applet to draw in/on.
     * @param time The time change since last draw.
     */
    public final void drawOn(final DDSApplet applet, final float time) {
        this.interpolate(time);
        this.drawOn(applet, time);
    }

    /**
     * The interpolate method handles making changes to this renderer's
     * positional data based on the last known motion data for this moveable
     * object. This will hopefully compensate for some lag that may occur
     * between updates from the model and the view.
     * 
     * @param time The time change since last draw.
     */
    public abstract void interpolate(final float time);

    /**
     * The basicInterpolate method is the simplest way to update a
     * MovableObjectRenderer's positional data based on the last known Kinetics.
     * 
     * @param time The time change since last draw.
     */
    public final void basicInterpolate(final float time) {
        this.viewKinetics.linearMotion(this.getPosition(), time);
    }

    /**
     * Constructor for AbstractMovableObjectRenderer.
     * 
     * @param obj The AbstractMovableObject that this object will render.
     */
    public AbstractMovableObjectRenderer(final AbstractMovableObject obj) {
        super(obj);
        this.viewKinetics = obj.getKinetics().duplicate();
    }

}
