package dds.view.simulation.renderers;

import saito.objloader.OBJModel;
import dds.control.DDSApplet;
import dds.model.simulation.AbstractSimObject;
import dds.model.simulation.GenericObject;
import dds.model.simulation.GenericObjectInterface;
import dds.view.simulation.AbstractSimObjectRenderer;

/**
 * @author Jacob Bolton
 * @author cyberdan
 */
public class GenericObjectRenderer extends AbstractSimObjectRenderer {

    /**
     * The .obj model.
     */
    private OBJModel model;

    /**
     * Getter for model.
     * 
     * @return the model
     */
    public final OBJModel getModel() {
        return model;
    }

    /**
     * Setter for model.
     * 
     * @param objmodel the model to set
     */
    public final void setModel(final OBJModel objmodel) {
        this.model = objmodel;
    }

    /**
     * The updateFromSim method handles updates coming from within the
     * simulation. Based on the update argument it will appropriately update the
     * view for the change in the model.
     * 
     * @param arg The argument for the update.
     * @see dds.view.simulation.AbstractSimObjectRenderer
     *      #updateFromSim(java.lang.Object)
     */

    public void updateFromSim(final Object arg) {
        // TODO Auto-generated method stub
    }

    /**
     * Constructor for GenericObjectRenderer.
     * 
     * @param obj The GenericObject that this instance wil be responsible for
     *            rendering.
     */
    public GenericObjectRenderer(final GenericObjectInterface obj) {
        super((AbstractSimObject) obj);
        this.model = obj.getModel3D();
    }

    /**
     * Constructor for GenericObjectRenderer.
     * 
     * @param obj The GenericObject that this instance wil be responsible for
     *            rendering.
     */
    public GenericObjectRenderer(final GenericObject obj) {
        this((GenericObjectInterface) obj);
    }

    /**
     * Constructor for GenericObjectRenderer.
     * 
     * @param obj The GenericObject that this instance wil be responsible for
     *            rendering.
     */
    public GenericObjectRenderer(final AbstractSimObject obj) {
        this((GenericObjectInterface) obj);
    }

    /**
     * The drawOn.
     * 
     * @param applet The applet on which to draw the context.
     */
    public final void drawModel(final DDSApplet applet) {
        applet.noStroke();
        // TODO Fix lighting so that materials don't have to rely on having
        // ambient values set.
        this.model.draw();
    }

    /**
     * The applyOrientation.
     * 
     * @param applet The applet to alter the transformation matrix of.
     * @see dds.view.simulation.AbstractSimObjectRenderer
     *      #applyOrientation(dds.control.DDSApplet)
     */
    @Override
    public void applyOrientation(final DDSApplet applet) {
        // TODO Auto-generated method stub
    }
}
