package dds.view.simulation.renderers;

import java.util.Collection;

import dds.control.DDSApplet;
import dds.model.simulation.AbstractSimObject;
import dds.model.simulation.RoadSign;
import dds.model.simulation.Vector3D;
import dds.view.simulation.AbstractSimObjectRenderer;

/**
 * The RoadSignRenderer class.
 * 
 * @author jbolton, cyberdan
 */
public class RoadSignRenderer extends AbstractSimObjectRenderer {

    /**
     * The STOP_SIGN is used to index the stop sign texture.
     */
    public static final int STOP_SIGN = 0;

    /**
     * The YEILD_SIGN is used to index the yeild sign texture.
     */
    public static final int YEILD_SIGN = 1;

    /**
     * The WARNING_SIGN is used to index the warning sign.
     */
    public static final int WARNING_SIGN = 2;

    /**
     * The OTHER is used to index the 'other' sign.
     */
    public static final int OTHER = 3;

    /**
     * The textures array stores file names for textures of the different types
     * of road signs.
     */
    public static final String[] TEXTURES = new String[] { "stopSign.jpg",
            "yeildSign.jpg", "warningSign.jpg", "otherSign.jpg" };

    // /**
    // * The NO_IMAGE constant is and easy-to-read way to set the texture image
    // to
    // * null.
    //     */
    //    private static final PImage NO_IMAGE = null;

    /**
     * The type of road sign.
     */
    private int type;

    /**
     * The label is the sign label to display.
     */
    private String label;

    /**
     * The verts is the collection vertices used to render the road sign.
     */
    private Collection<Vector3D> verts;

    // /**
    // * The textureImage is the image file to use for texturing this road sign.
    // */
    // private transient PImage textureImage;
    //
    /**
     * The accessor for the verts.
     * 
     * @return The vert collection.
     */
    public final Collection<Vector3D> getVerts() {
        return verts;
    }

    /**
     * The setVerts is the modifier for the verts.
     * 
     * @param vertCollection The verts to set.
     */
    public final void setVerts(final Collection<Vector3D> vertCollection) {
        this.verts = vertCollection;
    }

    /**
     * Getter for label.
     * 
     * @return the label
     */
    public final String getLabel() {
        return label;
    }

    /**
     * Setter for label.
     * 
     * @param labelstr the label to set
     */
    public final void setLabel(final String labelstr) {
        this.label = labelstr;
    }

    /**
     * Getter for type.
     * 
     * @return the type
     */
    public final int getType() {
        return type;
    }

    /**
     * Setter for type.
     * 
     * @param typenum the type to set
     */
    public final void setType(final int typenum) {
        this.type = typenum;
    }

    /**
     * The updateFromSim method handles updates coming from within the
     * simulation. Based on the update argument it will appropriately update the
     * view for the change in the model.
     * 
     * @param arg The argument for the update.
     * @see dds.view.simulation.AbstractSimObjectRenderer
     *      #updateFromSim(java.lang.Object)
     */

    public void updateFromSim(final Object arg) {
        // TODO Auto-generated method stub

    }

    /**
     * Constructor for RoadSignRenderer.
     * 
     * @param obj The RoadSign object that this object will render.
     */
    public RoadSignRenderer(final RoadSign obj) {
        super(obj);
        this.type = obj.getType();
        this.label = obj.getLabel();
        /*
         * this.textureImage = DDSApplet.getApplet()
         * .loadImage(TEXTURES[this.type]);
         */
        // this.textureImage = NO_IMAGE;
        if (obj.getVerts().size() == 0) {
            obj.calculateVerts();
        }
        this.setVerts(obj.getVerts());
    }

    /**
     * Constructor for RoadSignRenderer.
     * 
     * @param obj The RoadSign object that this object will render.
     */
    public RoadSignRenderer(final AbstractSimObject obj) {
        this((RoadSign) obj);
    }

    /**
     * The drawOn.
     * 
     * @param applet The applet on which to draw the context.
     */
    public final void drawModel(final DDSApplet applet) {
        /*
         * float deltaX = this.getPosition().getXpos(); float deltaY =
         * this.getPosition().getYpos(); float deltaZ =
         * this.getPosition().getZpos(); //applet.fill(255, 0, 0);
         * applet.beginShape(DDSApplet.POLYGON); if (textureImage != null) {
         * applet.texture(textureImage); } for (Vector3D v : this.getVerts()) {
         * applet.vertex(deltaX + v.getXpos(), deltaY + v.getYpos(), deltaZ +
         * v.getZpos()); } applet.endShape();
         */
    }

    /**
     * The applyOrientation.
     * 
     * @param applet The applet to work with.
     * @see dds.view.simulation.AbstractSimObjectRenderer
     *      #applyOrientation(dds.control.DDSApplet)
     */
    @Override
    public final void applyOrientation(final DDSApplet applet) {
        // TODO Determine what needs to go here.
    }
}
